from typing import List
from django.shortcuts import render, get_object_or_404
from django.core.exceptions import PermissionDenied
from django.contrib.auth.decorators import login_required
from django.http import HttpRequest, HttpResponse, Http404
from resources.models import (
    ProjectAccess,
    Collection,
    Project,
    Resource,
    Metadata,
    MetadataResourceValue,
)
from django.core.paginator import Paginator, EmptyPage
from django.views.decorators.cache import never_cache
from rpc.methods import _user_has_permission as user_has_permission
from django.contrib.auth.models import User
from rpc.const import PERM_RESOURCE_UPDATE

RESOURCES_PAGES_SIZE = 500


def editable_metadatas(project: Project) -> List[Metadata]:
    return (
        Metadata.objects.filter(project=project)
        .exclude(set__title__in=["ExifTool", "OCR"])
        .order_by("set", "title")
        .select_related("set")
    )


def user_has_project_access(user: User, project: Project) -> bool:
    if not ProjectAccess.objects.filter(project=project, user=user).first():
        raise PermissionDenied()


@login_required
@never_cache
def index(request: HttpRequest) -> HttpResponse:
    return render(
        request,
        "nui/index.html",
        {"accesses": ProjectAccess.objects.filter(user=request.user).iterator()},
    )


@login_required
@never_cache
def collection(request: HttpRequest, collection_id: int) -> HttpResponse:
    collection_instance: Collection = get_object_or_404(Collection, pk=collection_id)
    project_instance = collection_instance.project
    user_has_project_access(request.user, project_instance)
    resources = (
        collection_instance.resources.filter(deleted_at__isnull=True)
        .order_by("collectionmembership__rank", "title")
        .select_related("file")
    )
    resources_paginator = Paginator(resources, RESOURCES_PAGES_SIZE)
    try:
        page_number = int(request.GET.get("page", 1))
    except ValueError:
        page_number = 1
    try:
        resources_page = resources_paginator.page(page_number)
    except EmptyPage:
        resources_page = None
    return render(
        request,
        "nui/collection.html",
        {
            "collection": collection_instance,
            "project": project_instance,
            "resources_page": resources_page,
            "page_number": page_number,
        },
    )


def _fetch_resource(resource_id: int) -> Resource:
    return (
        Resource.objects.filter(pk=resource_id, deleted_at__isnull=True)
        .prefetch_related(
            "metadataresourcevalue_set",
            "metadataresourcevalue_set__metadata",
            "metadataresourcevalue_set__metadata__set",
        )
        .first()
    )


@login_required
@never_cache
def resource(
    request: HttpRequest, collection_id: int, resource_id: int
) -> HttpResponse:
    collection_instance: Collection = get_object_or_404(
        Collection, pk=collection_id, deleted_at__isnull=True
    )
    transient_message = ""
    resource_instance = _fetch_resource(resource_id)
    if not resource_instance:
        raise Http404()
    project_instance = resource_instance.ptr_project
    user_has_project_access(request.user, project_instance)
    user_has_update_permission = user_has_permission(
        request.user, project_instance, PERM_RESOURCE_UPDATE
    )
    if request.method == "POST" and user_has_update_permission:
        for key in request.POST.keys():
            new_title = request.POST.get("resource_title", "").strip()
            if new_title:
                resource_instance.title = new_title
                resource_instance.save()
            if "_" in key:
                parts = key.split("_")
                try:
                    int(parts[1])
                except ValueError:
                    continue
                if parts[0] == "addvalue":
                    metadata = Metadata.objects.get(
                        pk=parts[1], project=project_instance
                    )
                    for val in request.POST.getlist(key):
                        MetadataResourceValue.objects.get_or_create(
                            resource=resource_instance,
                            metadata=metadata,
                            value=val.strip(),
                        )
                if parts[0] == "changevalue":
                    new_val = request.POST.get(key).strip()
                    metaval = MetadataResourceValue.objects.filter(
                        pk=parts[1], resource=resource_instance
                    ).first()
                    if metaval and new_val != metaval.value:
                        metaval.value = new_val
                        metaval.save()
                if parts[0] == "deletevalue":
                    metaval = MetadataResourceValue.objects.get(
                        pk=parts[1], resource=resource_instance
                    )
                    metaval.delete()
        resource_instance = _fetch_resource(resource_id)  # reload metas
        transient_message = "Resource enregistrée"

    previous_resource = None
    next_resource = None
    ids_list = list(
        collection_instance.resources.filter(deleted_at__isnull=True)
        .order_by("collectionmembership__rank", "title")
        .values_list("id", flat=True)
    )
    for i in range(len(ids_list)):
        if ids_list[i] == resource_instance.pk:
            if ids_list[i - 1] != ids_list[-1]:
                previous_resource = Resource.objects.get(pk=ids_list[i - 1])
            if i < len(ids_list) - 1:
                next_resource = Resource.objects.get(pk=ids_list[i + 1])

    return render(
        request,
        "nui/resource.html",
        {
            "resource": resource_instance,
            "collection": collection_instance,
            "project": project_instance,
            "previous_resource": previous_resource,
            "next_resource": next_resource,
            "editable_metadatas": editable_metadatas(project_instance),
            "user_has_update_permission": user_has_update_permission,
            "transient_message": transient_message,
        },
    )


def search(request: HttpRequest) -> HttpResponse:
    collection_search = Collection.objects.filter(
        title__icontains=request.GET.get("q")
    ).order_by("title")
    resource_search = Resource.objects.filter(
        title__icontains=request.GET.get("q")
    ).order_by("title")
    return render(
        request,
        "nui/search.html",
        {"collection_search": collection_search, "resource_search": resource_search},
    )



from revproxy.views import ProxyView
from django.conf import settings
from django.shortcuts import redirect
from revproxy.response import get_django_response


def get_request_headers(self):
    """Return request headers that will be sent to upstream.

    The header REMOTE_USER is set to the current user
    if AuthenticationMiddleware is enabled and
    the view's add_remote_user property is True.

    .. versionadded:: 0.9.8

    If the view's add_x_forwarded property is True, the
    headers X-Forwarded-For and X-Forwarded-Proto are set to the
    IP address of the requestor and the request's protocol (http or https),
    respectively.

    .. versionadded:: TODO

    """
    request_headers = self.get_proxy_request_headers(self.request)

    if (
        self.add_remote_user
        and hasattr(self.request, "user")
        and self.request.user.is_active
    ):
        request_headers["REMOTE_USER"] = self.request.user.get_username()
        self.log.info("REMOTE_USER set")

    if self.add_x_forwarded:
        request_ip = self.request.META.get("REMOTE_ADDR")
        self.log.debug("Proxy request IP: %s", request_ip)
        request_headers["X-Forwarded-For"] = request_ip
        if settings.DEBUG:
            http_port = self.request.META["SERVER_PORT"]
            request_headers["X-Forwarded-Port"] = http_port
        request_proto = "https" if self.request.is_secure() else "http"
        self.log.debug("Proxy request using %s", request_proto)
        request_headers["X-Forwarded-Proto"] = request_proto
    return request_headers


ProxyView.get_request_headers = get_request_headers


class IIIFProxyView(ProxyView):
    upstream = settings.JAMA_IIIF_UPSTREAM_URL
    add_x_forwarded = True

    def dispatch(self, request, path):
        # print(f"### Upstream in setting: {settings.JAMA_IIIF_UPSTREAM_URL}")
        # print(f"### Upstream in class: {self.upstream}")
        # print(request.path)
        self.request_headers = self.get_request_headers()

        redirect_to = self._format_path_to_redirect(request)
        if redirect_to:
            return redirect(redirect_to)

        proxy_response = self._created_proxy_response(request, path)

        # print(proxy_response.headers)

        self._replace_host_on_redirect_location(request, proxy_response)
        self._set_content_type(request, proxy_response)

        response = get_django_response(
            proxy_response,
            strict_cookies=self.strict_cookies,
            streaming_amount=self.streaming_amount,
        )

        self.log.debug("RESPONSE RETURNED: %s", response)
        return response



from huey.contrib.djhuey import db_task, db_periodic_task
from huey import crontab
import logging
from typing import List
from django.conf import settings
from django.urls import reverse
from django.utils import timezone
from resources import models
from PIL import Image
import os
import time
import shutil
from PIL import UnidentifiedImageError
from PIL.Image import DecompressionBombError

logger = logging.getLogger(__name__)


@db_task()
def exif_task(file_id: int):
    from resources.models import File
    from resources.helpers import set_exif_metas

    try:
        f = File.objects.get(pk=file_id)
        set_exif_metas(f)
        if f.should_have_iiif:
            with Image.open(f.local_path()) as image:
                width, height = image.size
                f.denormalized_image_width = width
                f.denormalized_image_height = height
                f.save()

    except Exception as e:
        logger.warning("exit_task({}) failed".format(file_id))
        logger.info(repr(e))


@db_task(retries=2, retry_delay=600)
def iiif_task(file_id: int):
    from resources.models import File
    from resources.helpers import make_iiif

    try:
        f = File.objects.get(pk=file_id)
        make_iiif(f, True)
    except Exception as e:
        logger.warning("iiif_task({}) failed: {}".format(file_id, repr(e)))
        raise e


@db_task()
def ocr_task(file_id: int):
    from resources.models import File
    from resources.helpers import make_ocr

    try:
        f = File.objects.get(pk=file_id)
        make_ocr(f)
    except Exception as e:
        logger.warning("ocr_task({}) failed".format(file_id))
        logger.info(repr(e))


@db_task(retries=2, retry_delay=600)
def set_ark_to_resource(resource_id: int, location: str = None):
    if settings.ARK_SERVER and settings.ARK_APP_ID and settings.ARK_SECRET_KEY:
        from resources.models import Resource

        resource = Resource.objects.filter(pk=resource_id).first()
        if not resource:
            logger.warning(f"resource {resource_id} does not exist")
            return
        if resource.ark and not location:
            logger.info(f"resource {resource_id} has ark already, no update needed.")
            return
        import ark_client

        if not location:
            base_url = (
                settings.JAMA_SITE[:-1]
                if settings.JAMA_SITE[-1] == "/"
                else settings.JAMA_SITE
            )
            location = base_url + reverse(
                "ark_resource", kwargs={"resource_id": resource_id}
            )
        client = ark_client.Client(
            settings.ARK_APP_ID, settings.ARK_SECRET_KEY, settings.ARK_SERVER
        )
        if resource.ark:
            client.update(resource.ark, location)
        else:
            ark_name = client.create(location)
            resource.ark = ark_name
            resource.save()


@db_task(retries=2, retry_delay=600)
def set_ark_to_collection(collection_id: int, location: str = None):
    if settings.ARK_SERVER and settings.ARK_APP_ID and settings.ARK_SECRET_KEY:
        from resources.models import Collection

        collection = Collection.objects.filter(pk=collection_id).first()
        if not collection:
            logger.warning(f"collection {collection_id} does not exist")
            return
        if collection.ark and not location:
            logger.info(
                f"collection {collection_id} has ark already, no update needed."
            )
            return
        import ark_client

        if not location:
            base_url = (
                settings.JAMA_SITE[:-1]
                if settings.JAMA_SITE[-1] == "/"
                else settings.JAMA_SITE
            )
            location = base_url + reverse(
                "ark_collection", kwargs={"collection_id": collection_id}
            )
        client = ark_client.Client(
            settings.ARK_APP_ID, settings.ARK_SECRET_KEY, settings.ARK_SERVER
        )
        if collection.ark:
            client.update(collection.ark, location)
        else:
            ark_name = client.create(location)
            collection.ark = ark_name
            collection.save()


@db_task()
def recursive_set_metas_to_collection(
    user_id: int, collection_id: int, metas: List[dict], user_task_id: int = None
) -> bool:
    r"""
    Sets all metas for a unique metadata set.

    Metas is a list of metadata id => metadata value dictionaries.

    All metas must share the same metadata set.

    the meta will be set to all direct children resources.

    /!\ *Not* actually recursive: Descendants (sub-collections and sub-collections resources) are IGNORED.
    """
    from rpc.methods import (
        remove_meta_value_from_collection,
        add_meta_to_collection,
        set_metas_to_resource,
    )
    from resources.models import Metadata, Collection, MetadataSet, UserTask
    from django.contrib.auth.models import User

    user_task = UserTask.objects.filter(pk=user_task_id).first()
    if user_task:
        user_task.started_at = timezone.now()
        user_task.save()

    # prevent mixing metas from different sets
    metadatasets_ids = []
    for meta_dict in metas:
        meta = Metadata.objects.get(pk=meta_dict["id"])
        metadatasets_ids.append(meta.set.pk)
    metadatasets_ids = list(set(metadatasets_ids))
    if len(metadatasets_ids) > 1:
        if user_task:
            user_task.failed_at = timezone.now()
            user_task.save()
        return False

    # check that objects exist
    collection_instance = Collection.objects.filter(
        pk=collection_id, deleted_at__isnull=True
    ).first()
    if not collection_instance:
        if user_task:
            user_task.failed_at = timezone.now()
            user_task.save()
        return False
    metadataset_instance = MetadataSet.objects.filter(pk=metadatasets_ids[0]).first()
    if not metadataset_instance:
        if user_task:
            user_task.failed_at = timezone.now()
            user_task.save()
        return False

    user = User.objects.filter(pk=user_id).first()
    if not user:
        if user_task:
            user_task.failed_at = timezone.now()
            user_task.save()
        return False

    # remove all metas from the set
    for meta_value in collection_instance.metadatacollectionvalue_set.filter(
        metadata__set=metadataset_instance
    ):
        remove_meta_value_from_collection(user, collection_id, meta_value.pk)

    # add metas
    for meta_dict in metas:
        add_meta_to_collection(user, collection_id, meta_dict["id"], meta_dict["value"])

    for resource in collection_instance.resources.filter(deleted_at__isnull=True):
        set_metas_to_resource(user, resource.pk, metas)

    if user_task:
        user_task.finished_at = timezone.now()
        user_task.save()
    return True


@db_task()
def update_resources_from_xlsx_rows(
    user_id: int, xlsx_rows: List[dict], user_task_id: int = None
):
    from django.contrib.auth.models import User
    from resources.models import UserTask

    project = None
    if len(xlsx_rows) > 0:
        first_resource = models.Resource.objects.filter(
            pk=xlsx_rows[0].get("pk")
        ).first()
        if first_resource:
            project = first_resource.ptr_project
    user_task = None
    user = User.objects.filter(pk=user_id).first()
    if user:
        if user_task_id is not None:
            user_task = UserTask.objects.filter(pk=user_task_id).first()
            if user_task:
                user_task.project = project
                user_task.started_at = timezone.now()
                user_task.save()
        from rpc.methods import update_resource_from_xlsx_row, ServiceException

        for xlsx_row in xlsx_rows:
            try:
                update_resource_from_xlsx_row(user, xlsx_row)
            except ServiceException as e:
                logger.warning(e)
        if user_task:
            user_task.finished_at = timezone.now()
            user_task.save()


@db_periodic_task(crontab(hour="*/6"))
def ensureiiif():
    from resources.models import File
    from resources.helpers import iiif_destination_dir_from_hash

    start_date = timezone.now() - timezone.timedelta(days=1)
    for f in (
        File.objects.filter(
            file_type__serve_with_iiif=True,
            deleted_at__isnull=True,
            created_at__gte=start_date,
            tiled=False,
        )
        .distinct()
        .iterator()
    ):
        iiif_destination_dir = iiif_destination_dir_from_hash(f.hash)
        iiif_destination_file = "{}{}".format(iiif_destination_dir, f.hash)
        if not os.path.exists(iiif_destination_file):
            iiif_task(f.pk)
            logger.info(f"tasked missing IIIF for file({f.pk})")


@db_periodic_task(crontab(hour="*/6"))
def clean_partial_uploads():
    now = time.time()
    max_age = 60 * 60 * 6
    sub_folders = [
        f.path for f in os.scandir(settings.PARTIAL_UPLOADS_DIR) if f.is_dir()
    ]
    for folder in sub_folders:
        folder_time = os.path.getmtime(folder)
        if now - folder_time > max_age:
            logger.info(f"removing partial uploads dir {folder}")
            shutil.rmtree(folder)


@db_periodic_task(crontab(hour="*/6"))
def denorm_sizes():
    from resources.models import File

    start_date = timezone.now() - timezone.timedelta(days=1)
    for file in (
        File.objects.filter(
            denormalized_image_height__isnull=True,
            denormalized_image_width__isnull=True,
            file_type__serve_with_iiif=True,
            deleted_at__isnull=True,
            created_at__gte=start_date,
        )
        .distinct()
        .iterator()
    ):
        try:
            x = file.image_width()
            y = file.image_height()
            if x and y:
                file.save()
                logger.info(f"denormed sizes({x},{y}) for file({file.id})")
            else:
                logger.warning(f"can't find x and y for {file.pk}")
        except UnidentifiedImageError:
            logger.warning(f"can't identify {file.pk}")
        except DecompressionBombError:
            logger.warning(f"can't decompress {file.pk}")



# Generated by Django 5.1.7 on 2025-05-23 08:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("annotations", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="annotation",
            name="public",
            field=models.BooleanField(default=False),
        ),
    ]



from django.core.management.commands.shell import Command
from django.contrib.auth.models import User
from rich.console import Console
from rich.table import Table
from typing import List, Union
from resources.helpers import make_iiif, make_ocr
from django.utils.text import slugify
import secrets
import string

# from pick import pick
from resources.models import (
    Resource,
    Collection,
    Project,
    APIKey,
    ProjectAccess,
    Permission,
    File,
    ProjectProperty,
    UserTask,
    Tag,
    FileExtension,
    FileType,
    MetadataSet,
    Metadata,
    MetadataCollectionValue,
    MetadataResourceValue,
    CollectionMembership,
    Role,
)


def _console_table(title: str, headers: List["str"]) -> Table:
    table = Table(title=title)
    for header in headers:
        table.add_column(header)
    return table


def user(*args, **kwargs):
    return User.objects.get(*args, **kwargs)


def project(*args, **kwargs):
    return Project.objects.get(*args, **kwargs)


def projects(*args, **kwargs):
    table = _console_table(
        "Projects",
        [
            "pk",
            "label",
            "users",
        ],
    )

    for p in Project.objects.filter(*args, **kwargs):
        table.add_row(
            str(p.pk),
            str(p.label),
            ", ".join(
                f"{a.user.username} ({a.user.pk})"
                for a in ProjectAccess.objects.filter(project=p)
            ),
        )
    Console().print(table)


def users(*args, **kwargs):
    # option, index = pick(["some", "of", "this"], "choose...", multiselect=True)
    table = _console_table(
        "Users",
        ["pk", "username", "email", "active", "last login", "superuser", "projects"],
    )
    for u in User.objects.filter(*args, **kwargs).order_by("username"):
        table.add_row(
            str(u.pk),
            str(u.username),
            str(u.email),
            str(u.is_active),
            str(u.last_login),
            str(u.is_superuser),
            ", ".join(
                f"{a.project.label} ({a.project.pk})"
                for a in ProjectAccess.objects.filter(user=u)
            ),
        )
    Console().print(table)


def accesses():
    for a in ProjectAccess.objects.order_by("project"):
        print(a.pk, a)


def move_collection_to_project(from_collection_id: int, to_project_id: int):
    from resources.tasks import ocr_task

    from_collection_instance = Collection.objects.get(pk=from_collection_id)
    to_project_instance = Project.objects.get(pk=to_project_id)
    from_collection_instance.parent = to_project_instance.root_collection
    from_collection_instance.project = to_project_instance
    from_collection_instance.save()
    print(f"Collection({from_collection_instance.id})")
    for col in from_collection_instance.descendants():
        col.project = to_project_instance
        col.save()
        print(f"Collection({col.id})")
        for res in col.resources.iterator():
            res.ptr_project = to_project_instance
            res.save()
            print(
                "Metadatas deleted: ",
                MetadataResourceValue.objects.filter(resource=res).delete(),
            )
            print(f"Resource({res.id})")
            if res.file:
                res.file.project = to_project_instance
                res.file.save()
                print(f"File({res.file.id})")
                ocr_task(res.file.id)
    print("Done.")


def give_access(user: Union[str, int], project: Union[str, int], role: Union[str, int]):
    """
    Give access to user. Params can either be ints for object pks or strings for usernames and labels.
    """
    user_instance = None
    if isinstance(user, int):
        user_instance = User.objects.filter(pk=user).fist()
    if not user_instance:
        user_instance = User.objects.filter(username=user).first()
    if not user_instance:
        print(f'Utilisateur "{user}" introuvable')
    project_instance = None
    if isinstance(project, int):
        project_instance = Project.objects.filter(pk=project).fist()
    if not project_instance:
        project_instance = Project.objects.filter(label=project).first()
    if not project_instance:
        print(f'Projet "{project}" introuvable')
    role_instance = None
    if isinstance(role, int):
        role_instance = Role.objects.filter(project=project_instance, pk=role).first()
    if not role_instance:
        role_instance = Role.objects.filter(
            project=project_instance, label=role
        ).first()
    if not role_instance:
        print(f'Rôle "{role}" introuvable')
    if not role_instance or not user_instance or not project_instance:
        return
    return ProjectAccess.objects.get_or_create(
        role=role_instance, project=project_instance, user=user_instance
    )


def get_or_create_project(project_label: str) -> Project:
    """
    Get or creates a project based on given label.
    """
    project_label = project_label.strip()
    if not project_label:
        raise ValueError("needs a non-empty label")

    def add_dc_metas(project: Project, add_atlas_extras: bool = False):
        DC_METAS = [
            ["title", "A name given to the resource"],
            ["creator", "An entity primarily responsible for making the resource."],
            ["subject", "The topic of the resource."],
            ["description", "An account of the resource."],
            ["publisher", "An entity responsible for making the resource available."],
            [
                "contributor",
                "An entity responsible for making contributions to the resource.",
            ],
            [
                "date",
                "A point or period of time associated with an event in the lifecycle of the resource.",
            ],
            ["type", "The nature or genre of the resource."],
            [
                "format",
                "The file format, physical medium, or dimensions of the resource.",
            ],
            [
                "identifier",
                "An unambiguous reference to the resource within a given context.",
            ],
            [
                "source",
                "A related resource from which the described resource is derived.",
            ],
            ["language", "A language of the resource."],
            ["relation", "A related resource."],
            [
                "coverage",
                "The spatial or temporal topic of the resource, the spatial applicability of the resource, or the jurisdiction under which the resource is relevant.",
            ],
            ["rights", "Information about rights held in and over the resource."],
            [
                "audience",
                "A class of entity for whom the resource is intended or useful.",
            ],
            [
                "provenance",
                "A statement of any changes in ownership and custody of the resource since its creation that are significant for its authenticity, integrity, and interpretation.",
            ],
            [
                "rightsholder",
                "A person or organization owning or managing rights over the resource.",
            ],
        ]
        dc_set, _ = MetadataSet.objects.get_or_create(
            title="Dublin_Core", project=project
        )
        rank = 1
        for m in DC_METAS:
            meta, _ = Metadata.objects.get_or_create(
                title=m[0], set_id=dc_set.pk, project=project, rank=rank
            )
            rank = rank + 1
        return dc_set

    project = Project.objects.filter(label__iexact=project_label).first()
    if not project:
        project_admin_username = slugify(f"admin_{project_label.replace(' ', '_')}")
        project = Project.objects.create(label=project_label)
        admin_user, _ = User.objects.get_or_create(username=project_admin_username)
        admin_role, _ = Role.objects.get_or_create(label="admin", project=project)
        for permission in Permission.objects.all():
            admin_role.permissions.add(permission)
        ProjectAccess.objects.get_or_create(
            project=project, role=admin_role, user=admin_user
        )
        key, _ = APIKey.objects.get_or_create(
            user=admin_user,
            key="".join(
                secrets.choice(string.ascii_letters + string.digits) for _ in range(32)
            ),
        )
        key.active = True
        key.save()
        add_dc_metas(project)
        project.root_collection
    return project


def version():
    from jama import __version__ as v

    return v


def bpython(self, options):
    import bpython

    bpython.embed(
        {
            "APIKey": APIKey,
            "Collection": Collection,
            "File": File,
            "Permission": Permission,
            "Project": Project,
            "ProjectAccess": ProjectAccess,
            "Resource": Resource,
            "User": User,
            "ProjectProperty": ProjectProperty,
            "UserTask": UserTask,
            "Tag": Tag,
            "FileExtension": FileExtension,
            "FileType": FileType,
            "MetadataSet": MetadataSet,
            "Metadata": Metadata,
            "MetadataCollectionValue": MetadataCollectionValue,
            "MetadataResourceValue": MetadataResourceValue,
            "CollectionMembership": CollectionMembership,
            "Role": Role,
            "projects": projects,
            "users": users,
            "accesses": accesses,
            "make_iiif": make_iiif,
            "make_ocr": make_ocr,
            "give_access": give_access,
            "version": version,
            "get_or_create_project": get_or_create_project,
            # "move_collection_to_project": move_collection_to_project,
        }
    )


Command.bpython = bpython



# Generated by Django 5.1.7 on 2025-04-30 15:36

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0039_alter_project_use_exiftool"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="metadata",
            name="data_type",
        ),
        migrations.RemoveField(
            model_name="resourceclass",
            name="set",
        ),
        migrations.RemoveField(
            model_name="resource",
            name="resource_class",
        ),
        migrations.DeleteModel(
            name="MetadataType",
        ),
        migrations.DeleteModel(
            name="ResourceClass",
        ),
    ]



# Generated by Django 5.1.7 on 2025-05-14 19:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "resources",
            "0040_remove_metadata_data_type_remove_resourceclass_set_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="metadata",
            options={
                "ordering": ["set", "rank", "title"],
                "verbose_name": "meta-donnée",
                "verbose_name_plural": "meta-données",
            },
        ),
        migrations.AlterModelOptions(
            name="metadataresourcevalue",
            options={"ordering": ("metadata",)},
        ),
    ]



# Generated by Django 3.2.2 on 2021-10-20 08:44

from django.db import migrations


def fixtures(*args):
    pass  # obsolete fixtures, removed


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0021_auto_20211007_1748"),
    ]

    operations = [migrations.RunPython(fixtures)]



# Generated by Django 5.2.5 on 2025-09-09 18:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("resources", "0041_alter_metadata_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="collection",
            name="published_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from django.urls import path

from . import views

urlpatterns = [
    path(
        "thumb/<str:sha256>/rotate/<str:angle>/scale/<str:scale>/crop/<str:crop>",
        views.thumb,
        name="picthumb",
    ),
    path(
        "simple_thumb/<str:sha256>/<int:size>", views.simple_thumb, name="simplethumb"
    ),
]



from django.urls import path

from . import views

app_name = "nui"
urlpatterns = [
    path("", views.index, name="index"),
    path("collection/<int:collection_id>", views.collection, name="collection"),
    path(
        "collection/<int:collection_id>/resource/<int:resource_id>",
        views.resource,
        name="resource",
    ),
    path("search", views.search, name="search"),
    # path("test", views.test),
]
